<?php

/*************************************************************************/
/* DOCEBO LCMS - Learning Content Managment System                       */
/* ======================================================================*/
/* Docebo is the new name of SpaghettiLearning Project                   */
/*                                                                       */
/* Copyright (c) 2005 by Emanuele Sandri (esandri@tiscali.it)            */
/*                                                                       */
/* This program is free software. You can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation; either version 2 of the License.        */
/*************************************************************************/

if(!defined('IN_DOCEBO')) die('You cannot access this file directly');

require_once( dirname(__FILE__) .'/lib.aclmanager.php' );

/**
 * Acl common tasks class
 * This class is only for check permissions
 * To manage ACLs we must use DoceboACLManager
 *
 * NOTE: ST means Security Token
 * @package  DoceboCore
 * @version  $Id: lib.acl.php 419 2006-06-01 16:36:00Z fabio $
 * @category ACL
 * @author   Emanuele Sandri <esandri@tiscali.it>
 */
class DoceboACL {
	/** Instance of DoceboACLManager */
	var $aclManager = NULL;
	
	/**
	 * constructor
	 * @param mixed $dbconn the connection to database or FALSE to use default connection
	 * @param mixed $prefix the prefix of the database or FLASE to use default prefix
	 */
	function DoceboACL( $dbconn = FALSE, $prefix = FALSE ) {
		$this->aclManager = new DoceboACLManager($dbconn, $prefix );
	}
	
	function &getACLManager() {
		return $this->aclManager;
	}
	
	/* NOTE: functions for retrieve single security token */
	/**
	 * get security token of an user
	 * @param string $userid id of the user
	 * @return mixed security token associated to user or FALSE
	 **/
	function getUserST( $userid ) {
		$arr = $this->aclManager->getUser( FALSE, $userid );
		if( $arr === FALSE )
			return FALSE;
		return $arr[ACL_INFO_IDST];
	}
	
	/**
	 * get security token of a group
	 * @param string $groupid id of the group
	 * @return mixed security token associated to groups or FALSE
	 **/
	function getGroupST( $groupid ) {
		$arr = $this->aclManager->getGroup( FALSE, $groupid );
		if( $arr === FALSE )
			return FALSE; 
		
		return $arr[ACL_INFO_IDST];
	}
	
	/**
	 * get security token of a role
	 * @param string $roleid id of the role
	 * @return mixed security token associated to role or FALSE
	 **/
	function getRoleST( $roleid ) {
		$arr = $this->aclManager->getRole( FALSE, $roleid );
		if( $arr === FALSE )
			return FALSE; 
		return $arr[ACL_INFO_IDST];
	}
	
	/* NOTE: functions for retrieve all security token */
	/**
	 * get all security token of a user
	 * @param string $userid id of the user
	 * @param string $filter (Optional). Filter to applay.
	 * @return mixed array of security tokens associated to the user or FALSE
	 **/
	function getUserAllST( $userid, $filter = '' ) {
		$arrST[] = $this->getUserST( $userid );
		
		// search in groups		
		$count = 0;
		while( $count < count( $arrST ) ) {
			
			$idST = $arrST[$count];
			$arrResult = $this->aclManager->getGroupsContainer( $idST, $filter );
			$arrST = array_merge( $arrST, array_diff($arrResult, $arrST ));
			$count++;
		}
		
		// search in roles
		$arrRoles = array();
		$count = 0;
		while( $count < count( $arrST ) ) {
			
			$idST = $arrST[$count];
			$arrResult = $this->aclManager->getRolesContainer( $idST );
			$arrRoles = array_merge( $arrRoles, array_diff($arrResult, $arrRoles ));
			$count++;
		}
		$arrST = array_merge( $arrST, $arrRoles);
		
		return $arrST;
	}
	
	/**
	 * get all security token of a group
	 * @param string $groupid id of the group
	 * @param string $filter (Optional). Filter to applay.
	 * @return mixed array of security tokens associated to the group or FALSE
	 **/
	function getGroupAllST( $groupid, $filter = '' ) {
		$arrST[] = $this->getGroupST( $groupid );
		// search in groups
		$count = 0;
		while( $count < count( $arrST ) ) {
			$idST = $arrST[$count];
			$arrResult = $this->aclManager->getGroupsContainer( $idST, $filter );
			$arrST = array_merge( $arrST, array_diff($arrResult, $arrST ));
			$count++;
		}
		// search in roles
		$arrRoles = array();
		$count = 0;
		while( $count < count( $arrST ) ) {
			$idST = $arrST[$count];
			$arrResult = $this->aclManager->getRolesContainer( $idST );
			$arrRoles = array_merge( $arrRoles, array_diff($arrResult, $arrRoles ));
			$count++;
		}
		$arrST = array_merge( $arrST, $arrRoles);
		return $arrST;
	}
	
	/* NOTE: functions for retrieve groups */
	/**
	 * get all groups of a idst
	 * @param int $idst idst
	 * @param string $filter (Optional). Filter to applay.
	 * @return mixed array of groups of the user or FALSE
	 **/
	function getSTGroupsST( $idst, $filter = '') {
		$arrST = $this->aclManager->getGroupsContainer( $idst, $filter );
		// search in groups		
		$count = 0;
		while( $count < count( $arrST ) ) {
			$idST = $arrST[$count];
			$arrResult = $this->aclManager->getGroupsContainer( $idST, $filter );
			$arrST = array_merge( $arrST, array_diff($arrResult, $arrST ));
			$count++;
		}
		return $arrST;
	}
	
	/**
	 * get all groups of an array of idst
	 * @param int $arr_idst idst
	 * @param string $filter (Optional). Filter to applay.
	 * @return mixed array of groups of the user or FALSE
	 **/
	function getArrSTGroupsST( $arr_idst, $filter = '' ) {
		$arrST = array_values($arr_idst);
		// search in groups		
		$count = 0;
		while( $count < count( $arrST ) ) {
			$idST = $arrST[$count];
			$arrResult = $this->aclManager->getGroupsContainer( $idST, $filter );
			$arrST = array_values(array_unique(array_merge( $arrST, array_diff($arrResult, $arrST ))));
			$count++;
		}
		return $arrST;
	}
	
	/**
	 * get all groups of a idst_user
	 * @param int $idst_user idst
	 * @param string $filter (Optional). Filter to applay.
	 * @return mixed array of groups of the user or FALSE
	 **/
	function getUserGroupsST( $idst_user, $filter = '' ) {
		return $this->getSTGroupsST( $idst_user, $filter );
	}
	
	/**
	 * get all groups of a idst_group
	 * @param int $idst_group idst
	 * @param string $filter (Optional). Filter to applay.
	 * @return mixed array of groups of the user or FALSE
	 **/
	function getGroupGroupsST( $idst_group, $filter = '' ) {
		return $this->getSTGroupsST( $idst_group, $filter );
	}
	
	/**
	 * get all groups of a group
	 * @param int $idst idst of the group
	 * @return mixed array of groups of the group or FALSE
	 **/
	function getGroupGDescendants( $idst ) {
		return $this->aclManager->getGroupGDescendants( $idst );
	}
	
	/**
	 * get all descendant users of a group 
	 * @param int $idst idst of the group
	 * @return mixed array of users of the group or FALSE
	 **/
	function getGroupUDescendants( $idst ) {
		return $this->aclManager->getGroupUDescendants( $idst );
	}
	
	/* NOTE: functions for retrieve roles */
	/**
	 * get all roles of a user
	 * @param string $userid id of the user
	 * @return mixed array of roles of the user or FALSE
	 **/
	function getUserRoles( $userid ) {
	}
	
	/**
	 * get all roles of a group
	 * @param string $groupid id of the group
	 * @return mixed array of roles of the group or FALSE
	 **/
	function getGroupRoles( $groupid ) {
	}
	
	/* NOTE: functions to test match with single security token */
	function _searchMatch( $idst, $idstMatch ) {
		$arrST[] = $idst;
		if( in_array($idst,$idstMatch ) )
			return TRUE;
		// search in groups
		$idST = current( $arrST );
		while( $idST !== FALSE ) {
			$arrResult = $this->aclManager->getGroupsContainer( $idST );
			$arrDiff = array_diff($arrResult, $arrST );
			if( count(array_intersect($idstMatch,$arrDiff)) > 0 )
				return TRUE;
			$arrST = array_merge( $arrST, $arrDiff);			
			$idST = next( $arrST );
		}
		reset( $arrST );
		// search in roles
		$arrRoles = array();
		$idST = current( $arrST );
		while( $idST !== FALSE ) {
			$arrResult = $this->aclManager->getRolesContainer( $idST );
			$arrDiff = array_diff($arrResult, $arrST );
			if( count(array_intersect($idstMatch,$arrDiff)) > 0 )
				return TRUE;
			$arrRoles = array_merge( $arrRoles, array_diff($arrResult, $arrRoles ));
			$idST = next( $arrST );
		}
		return FALSE;		
	}
	
	/**
	 * match a user with a security token
	 * @param string $userid id of the user
	 * @param int $st security token to match
	 * @return bool TRUE if match, FALSE otherwise
	 **/
	function matchUserST( $userid, $st ) {
		$idst = $this->getUserST( $userid );
		return $this->_searchMatch( $idst, array($st) );
	}
	
	/**
	 * match a group with a security token
	 * @param string $groupid id of the group
	 * @param int $st security token to match
	 * @return bool TRUE if match, FALSE otherwise
	 **/
	function matchGroupST( $groupid, $st ) {
		$idst = $this->getGroupST( $groupid );
		return $this->_searchMatch( $idst, array($st) );
	}
	
	/* NOTE: functions to test match with multiple security token */
	/**
	 * match a user with an array of security token
	 * @param string $userid id of the user
	 * @param array $st array of security token to match
	 * @return bool TRUE if match, FALSE otherwise
	 **/
	function matchUserSTArray( $userid, $st ) {
		$idst = $this->getUserST( $userid );
		return $this->_searchMatch( $idst, $st );
	}
	
	/**
	 * match a group with an array of security token
	 * @param string $groupid id of the group
	 * @param int $st array of security token to match
	 * @return bool TRUE if match, FALSE otherwise
	 **/
	function matchGroupSTArray( $groupid, $st ) {
		$idst = $this->getGroupST( $groupid );
		return $this->_searchMatch( $idst, $st );
	}
}

?>
