<?php
/*************************************************************************/
/* DOCEBO CORE - Framework                                               */
/* ============================================                          */
/*                                                                       */
/* Copyright (c) 2006 by Giovanni Derks <virtualdarkness[AT]gmail-com>   */
/* http://www.docebo.com                                                 */
/*                                                                       */
/* This program is free software. You can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation; either version 2 of the License.        */
/*************************************************************************/

/**
 * @package  DoceboCore
 * @category UrlManager
 * @version  $Id: lib.urlmanager.php 131 2006-03-16 15:09:20Z giovanni $
 */





class UrlManager {

	var $use_mod_rewrite=FALSE;
	var $mod_rewrite_title=NULL;
	var $mod_rewrite_url_items=array();
	var $mod_rewrite_url_pattern=NULL;

	var $std_query=NULL;
	var $query_items=array();
	var $query_map=FALSE;
	var $ignore_items=array();

	var $std_base_url="index.php";

	/**
	 * UrlManager constructor
	 */
	function UrlManager() {

	}


	/**
	 *
	 */
	function setBaseUrl($url) {
		$this->std_base_url=$url;
	}


	/**
	 *
	 */
	function getBaseUrl() {
		return $this->std_base_url;
	}


	/**
	 *
	 */
	function setQueryItems($items) {

		$ignore=$this->getIgnoreItems();
		if ((is_array($ignore)) && (count($ignore) > 0)) {
			foreach ($ignore as $item_key) {
				if (isset($items[$item_key]))
					unset($items[$item_key]);
			}
		}

		if ($this->getQueryMap() !== FALSE)
			$items=$this->applyQueryMap($items);

		$this->query_items=$items;
	}


	/**
	 *
	 */
	function getQueryItems() {
		return (array)$this->query_items;
	}


	/**
	 *
	 */
	function setStdQuery($query) {
		$res=array();

		if (is_array($query)) {
			$res=$this->explodeQueryItems($query);
		}
		else if (is_string($query)) {
			$items=explode("&", trim($query, "&"));
			$res=$this->explodeQueryItems($items);
		}

		if ($this->getQueryMap() !== FALSE)
			$res=$this->applyQueryMap($res);

		$this->std_query=$res;
	}


	/**
	 *
	 */
	function getStdQuery() {
		return (array)$this->std_query;
	}


	/**
	 *
	 */
	function setQueryMap($array) {
		$this->query_map=$array;
	}


	/**
	 *
	 */
	function applyQueryMap($query) {

		//$query_items=$this->explodeQueryItems($query);
		$query_items=$query;

		foreach ($this->getQueryMap() as $key=>$val) {

			if (array_key_exists($val, $query_items))
				$this->printError("You are trying to map a key to another one that already exists: ".$key." -&gt; ".$val);

			if (isset($query_items[$key])) {
				$query_items[$val]=$query_items[$key];
				unset($query_items[$key]);
			}
		}
		ksort($query_items);
		//$res=$this->implodeQueryItems($query_items);
		$res=$query_items;
		return $res;
	}


	/**
	 *
	 */
	function disableQueryMap() {
		$this->query_map=FALSE;
	}


	/**
	 *
	 */
	function getQueryMap() {
		return $this->query_map;
	}


	/**
	 *
	 */
	function setIgnoreItems($array) {
		$this->ignore_items=$array;
	}


	/**
	 * Used only on query items; not on standard items cause it won't make sense
	 */
	function getIgnoreItems() {
		return $this->ignore_items;
	}


	/**
	 *
	 */
	function setUseModRewrite($use) {
		$this->use_mod_rewrite=$use;
	}


	/**
	 *
	 */
	function getUseModRewrite() {
		return (bool)$this->use_mod_rewrite;
	}


	/**
	 *
	 */
	function setModRewriteTitle($txt) {
		$this->mod_rewrite_title=$txt;
	}


	/**
	 *
	 */
	function getModRewriteTitle() {
		return $this->mod_rewrite_title;
	}


	/**
	 *
	 */
	function setModRewriteUrlItems($items) {
		$this->mod_rewrite_url_items=$items;
	}


	/**
	 *
	 */
	function getModRewriteUrlItems() {
		return (array)$this->mod_rewrite_url_items;
	}


	/**
	 *
	 */
	function setModRewriteUrlPattern($pattern) {
		$this->mod_rewrite_url_pattern=$pattern;
	}


	/**
	 *
	 */
	function getModRewriteUrlPattern() {
		return $this->mod_rewrite_url_pattern;
	}


	/**
	 *
	 */
	function getUrl($query=FALSE, $use_html_code=TRUE) {
		$res="";

		if ($use_html_code)
			$amp="&amp;";
		else {
			$amp="&";
		}

		if (is_array($query)) {
			$this->setQueryItems($this->explodeQueryItems($query));
		}
		else if (is_string($query)) {
			$items=explode("&", trim($query, "&"));
			$this->setQueryItems($this->explodeQueryItems($items));
		}
		else {
			$this->setQueryItems(array());
		}

		if (!$this->getUseModRewrite()) { // mod_rewrite OFF

			$res.=$this->getBaseUrl();
			$url_query="";

			$query_items=array_merge($this->getStdQuery(), $this->getQueryItems());
			$query_items=$this->implodeQueryItems($query_items);

			$url_query.=implode($amp, $query_items);

			if (!empty($url_query))
				$res.="?".$url_query;

		}
		else {  // mod_rewrite ON

			$res=$this->getModRewriteUrl();

		}

		return $res;
	}


	/**
	 *
	 */
	function explodeQueryItems($query) {
		$other_items=array();
		foreach ($query as $val) {
			$current_item=explode("=", $val);
			if (count($current_item) > 1)
				$other_items[$current_item[0]]=$current_item[1];
		}
		return $other_items;
	}


	/**
	 *
	 */
	function implodeQueryItems($query_items) {
		$query=array();
		foreach ($query_items as $key=>$val) {
			$query[]=$key."=".$val;
		}
		return $query;
	}


	/**
	 *
	 */
	function getModRewriteUrl() {
		$res="";

		$pattern=$this->getModRewriteUrlPattern();
		$items=$this->getModRewriteUrlItems();
		$other_items_list=array_merge($this->getStdQuery(), $this->getQueryItems());
		$title=$this->getModRewriteTitle();

		//$other_items=$this->explodeQueryItems($other_items_list);
		$other_items=$other_items_list;

		$res=$pattern;

		if ((preg_match("/\\[T\\]/", $pattern)) && (empty($title)))
			$this->printError("Your pattern requires a title but it has not been set.");
		else if ((preg_match("/\\[T\\]/", $pattern)) && (!empty($title)))
			$res=preg_replace("/\\[T\\]/", $title, $res);

		$req_param_count=substr_count($pattern, "[P]");

		if (count($items) < $req_param_count)
			$this->printError("Your pattern requires more parameters. (".count($items)." < ".$req_param_count.")");

		foreach($items as $item_key) {

			if (isset($other_items[$item_key])) {
				$replace=$other_items[$item_key];
				unset($other_items[$item_key]);
			}
			else
				$replace=$item_key."-NOT-SET";

			$res=preg_replace("/\\[P\\]/", $replace, $res, 1);

		}

		if (preg_match("/\\/\\[O\\]\\//", $pattern)) { // Other parameters

			if (count($other_items) < 1) {
				$other_items_str="/0/";
			}
			else {
				$other_items_str=$this->getOtherItemsStr($other_items);
			}
			$res=preg_replace("/\\/\\[O\\]\\//", $other_items_str, $res);
		}

		return $res;
	}


	/**
	 *
	 */
	function getOtherItemsStr($items) {
		$from=array("-", "_");
		$to=array("--", "__");

		$mr_arr=array();
		foreach($items as $key=>$val) {
			$my_key=str_replace($from, $to, $key);
			$my_val=str_replace($from, $to, $val);

			if (!empty($my_val))
				$mr_arr[]=$my_key."_".$my_val;
		}

		$res="/".rawurlencode(implode("-", $mr_arr))."/";
		return $res;
	}


	/**
	 *
	 */
	function loadOtherModRewriteParamFromVar($mr_str) {

		if (($this->getUseModRewrite()) && (!empty($mr_str))) {

			$mr_str=rawurldecode($mr_str);

			$mr_arr=$this->getCleanMrArray($mr_str, "-");

			foreach ($mr_arr as $key=>$val) {
				$val_arr=$this->getCleanMrArray($val, "_");

				$my_key=$val_arr[0];
				$my_val=$val_arr[1];

				if ((!isset($_GET[$my_key])) && (!empty($my_val))) {
					$_GET[$my_key]=$my_val;
					//-debug:-// echo("<br />"."\$_GET[".$my_key."]=".$my_val);
				}
			}

		}

	}


	/**
	 *
	 */
	function getCleanMrArray($mr_str, $sep) {
		$mr_arr=array();

		if (preg_match("/[".$sep."]{2,2}/", $mr_str)) { // Optimized ;)

			$mr_str_map=preg_replace("/[".$sep."]{2,2}/", "  ", $mr_str);
			$mr_str_map_arr=preg_split("/".$sep."/", $mr_str_map, -1, PREG_SPLIT_OFFSET_CAPTURE);

			$mr_arr=$this->splitFromMap($mr_str, $mr_str_map_arr, $sep);

		}
		else {
			$mr_arr=explode($sep, $mr_str);
		}

		return $mr_arr;
	}


	/**
	 *
	 */
	function splitFromMap($str, $arr, $sep) {
		$res=array();
		$_OFFSET=1;

		$i=0;
		while($i < count($arr)) {

			if ($i+1 < count($arr))
				$res[$i]=substr($str, $arr[$i][$_OFFSET], ($arr[$i+1][$_OFFSET]-$arr[$i][$_OFFSET]-1));
			else
				$res[$i]=substr($str, $arr[$i][$_OFFSET]);

			$res[$i]=preg_replace("/[".$sep."]{2,2}/", $sep, $res[$i]);

			$i++;
		}

		return $res;
	}


	/**
	 *
	 */
	function printError($txt) {
		if ($GLOBALS["framework"]["do_debug"] == "on")
			die($txt);
	}


}


?>
