<?php

/************************************************************************/
/* DOCEBO CORE - Framework												*/
/* ============================================							*/
/*																		*/
/* Copyright (c) 2005													*/
/* http://www.docebo.com												*/
/*																		*/
/* This program is free software. You can redistribute it and/or modify	*/
/* it under the terms of the GNU General Public License as published by	*/
/* the Free Software Foundation; either version 2 of the License.		*/
/************************************************************************/

require_once( dirname(__FILE__).'/lib.event.php' );

/**
 * @package  DoceboCore-Event
 * @version  $Id: lib.usernotifier.php 261 2006-04-14 13:30:13Z fabio $
 * @category Event
 * @author   Emanuele Sandri <esandri@docebo.com>
 *
 * This is the class for ClassEvents in Docebo
**/
class DoceboUserNotifier extends DoceboEventConsumer {

	function _getConsumerName() {
		return "DoceboUserNotifier";
	}

	function actionEvent( &$event ) {
		
		parent::actionEvent($event);
		
		// initializing		
		require_once($GLOBALS["where_framework"]."/setting.php");
		require_once($GLOBALS["where_framework"]."/lib/lib.field.php");
		
		$acl_man =& $GLOBALS['current_user']->getACLManager();
		$field_man = new FieldList();
		$send_to_field = $GLOBALS['framework']['sms_cell_num_field'];
		
		$arr_mail_recipients = array();
		$arr_sms_recipients = array();
		
		// recover event information
		$arr_recipients 	= explode( ',', $event->getProperty('recipientid') );
		$msg_composer 		= unserialize(urldecode($event->getProperty('MessageComposer')));
		$force_email_send 	= $event->getProperty('force_email_send');
		
		if(!is_array($arr_recipients) || empty($arr_recipients)) return;
		
		// recover user info and convert to idst if required
		if(is_numeric($arr_recipients[0])) $idst_users =& $arr_recipients;
		else $idst_users = $acl_man->fromUseridToIdst($arr_recipients);
		$users_info = $acl_man->getUsers($idst_users);
		
		// recove setting 
		$users_lang		= $acl_man->getSettingValueOfUsers('ui.language', false, true);
		$users_sms 		= $field_man->showFieldForUserArr($idst_users, array($send_to_field));
		
		// scan all users
		
		if(!is_array($users_info) || empty($users_info)) return;
		while(list(, $user_dett) = each($users_info)) {
			
			// recover media setting
			$idst_user = $user_dett[ACL_INFO_IDST];
			$media = usernotifier_getUserEventChannel( $idst_user, $event->getClassName() );
			
			$lang = ( isset($users_lang[$idst_user]) && $users_lang[$idst_user] !== NULL 
				? $users_lang[$idst_user] 
				: getDefaultLanguage() );
			
			if(in_array('email', $media) || $force_email_send == 'true') 
				if($user_dett[ACL_INFO_EMAIL] != '') 
					$arr_mail_recipients[$lang][$idst_user] = $user_dett[ACL_INFO_EMAIL];
			
			if(in_array('sms', $media)) 
				if($users_sms[$idst_user][$send_to_field] != '') 
					$arr_sms_recipients[$lang][$idst_user] = $users_sms[$idst_user][$send_to_field];
			
		}
		
		if(!empty($arr_mail_recipients)) {
			
			$lang_mail = array_keys($arr_mail_recipients);
			foreach($lang_mail as $lang_code) {
				
				reset($arr_mail_recipients[$lang]);
				$this->_sendMail(	$msg_composer->getSubject('email', $lang_code), 
									$msg_composer->getBody('email', $lang_code), 
									$arr_mail_recipients[$lang] );
			}
		}
		if(!empty($arr_sms_recipients)) {
			
			$lang_sms = array_keys($arr_sms_recipients);
			foreach($lang_sms as $lang_code) {
				
				reset($arr_sms_recipients[$lang_sms]);
				$this->_sendSms(	$msg_composer->getBody('sms', $lang_code),
									$arr_sms_recipients[$lang_code] );
			}
		}
	}
	
	function _sendMail($subject, $body, &$mail_recipients) {
		
		// prepare intestation for email
		$from = "From: ".$GLOBALS['framework']['sender_event']."\r\n";
		$intestazione  = "MIME-Version: 1.0\r\nContent-type: text/html; charset=".getUnicode()."\r\n";
		$intestazione .= "Return-Path: ".$GLOBALS['framework']['sender_event']."\r\n";
		$intestazione .= "Reply-To: ".$GLOBALS['framework']['sender_event']."\r\n";
		$intestazione .= "X-Sender: ".$GLOBALS['framework']['sender_event']."\r\n";
		$intestazione .= "X-Mailer: PHP/". phpversion()."\r\n";
		
		// send mail
		while(list(, $recipient) = each($mail_recipients)) {
			
			@mail($recipient , stripslashes($subject), stripslashes(nl2br($body)), $from.$intestazione."\r\n");
		}
	}
	
	function _sendSms($body, &$sms_recipients) {
		
		// in the future, if we'll have more than one gateway, smsmarket
		// will became a class that will include all the smsmarket's methods
		// and we will just load the class and call $obj->sendsms(..)
		
		switch($GLOBALS['framework']['sms_gateway']) {
			case "smsmarket": {
				$this->_smsmarket_Send(strip_tags($body), $sms_recipients);
			} break;
		}
	}
	
	function _smsmarket_Send($body, $arr_smslist) {
		
		
		// initialize
		$sent 		= 0;
		$failed 	= 0;
		$flash 		= false;
		$sender 	= $GLOBALS['framework']["sms_sent_from"];
		$errno 		= NULL;
		$errstr 	= NULL;
		
		// Connect
		$socket_conn = fsockopen($GLOBALS['framework']["sms_gateway_host"], $GLOBALS['framework']["sms_gateway_port"], $errno, $errstr, 10);
		if(!$socket_conn) return false;
		
		// Login
		fputs($socket_conn, $this->smsmarket_Wrap("\x06".$GLOBALS['framework']["sms_gateway_user"]."\x04".$GLOBALS['framework']["sms_gateway_pass"]."\x04"));
		$response 	= fgets($socket_conn, 128);
		$connected 	= ( ereg("Ok",$response) ? true : false );
		
		
		// not enought credit
		if($GLOBALS['framework']["sms_credit"] < 0) {
			
			// if not enoght credit try to send an sms to refresh the credit
			$cell_num = array_pop($arr_smslist);
			
			if($cell_num != '' && $cell_num != NULL && $cell_num != false) {
				
				if (!preg_match("/^\\+/", $cell_num))
					$cell_num = $GLOBALS['framework']["sms_international_prefix"].$cell_num;
				
				// send sms
				$str = $this->smsmarket_Wrap($this->smsmarket_Msg(	$body, 
																		$cell_num, 
																		$sender, 
																		$flash, 
																		"myfile.sms"));
				
				// check server answer 
				fputs($socket_conn, $str);
				$response = fgets($socket_conn, 128);
				if(ereg("Ok",$response)) $sent++;
				else $failed++;
			}
			
			// retrive credit
			$cr = array();
			eregi("Remaining:([0-9]+)", $response, $cr);
			$credits_left = $cr[1];
			
			if($credits_left < 0) return false;
			else {
				
				// save credit value
				$query_credit = "
				UPDATE ".$GLOBALS['prefix_fw']."_setting 
				SET param_value='".$credits_left."' 
				WHERE param_name='sms_credit'";
				$re_credit = mysql_query($query_credit);
			}
		}
		
		//Send all the sms
		while(list(, $cell_num) = each($arr_smslist)) {
			
			// validate sms dest number
			if($cell_num != '' && $cell_num != NULL && $cell_num != false) {
				
				if (!preg_match("/^\\+/", $cell_num))
					$cell_num = $GLOBALS['framework']["sms_international_prefix"].$cell_num;
				
				// send sms
				$str = $this->smsmarket_Wrap($this->smsmarket_Msg(	$body, 
																		$cell_num, 
																		$sender, 
																		$flash, 
																		"myfile.sms"));
				
				// check server answer 
				fputs($socket_conn, $str);
				$response = fgets($socket_conn, 128);
				if(ereg("Ok",$response)) $sent++;
				else $failed++;
			}
		}
		if($GLOBALS['do_debug'] == 'on') $GLOBALS['page']->add('<!-- send sms: '.$sent.' - '.$failed.'; -->', 'debug');
		// retrive credit
		$cr = array();
		eregi("Remaining:([0-9]+)", $response, $cr);
		$credits_left = $cr[1];
		
		// save credit value
		$query_credit = "
		UPDATE ".$GLOBALS['prefix_fw']."_setting 
		SET param_value='".$credits_left."' 
		WHERE param_name='sms_credit'";
		$re_credit = mysql_query($query_credit);
		
		// close connection and return
		fclose($socket_conn);
		return true;
	}
	
	function smsmarket_Wrap($data) {
		
		$chk = 0;
		for($i = 0; $i < strlen($data); $i++) $chk += ord(substr($data,$i,1));
		$chk = sprintf("%02X",$chk % 256);
		return "\x0B".$data.$chk."\x03";
	}
	
	function smsmarket_Msg($text,$dest,$sender,$flash,$type) {
		
		return "\x05".$dest."\x04".$sender."\x04".$text."\x04".$flash."\x04".$type."\x04".(int)$GLOBALS["sms_gateway_id"]."\x04";
	}
	
}

function usernotifier_getUserEventChannel( $idst, $event_class ) {
	
	$query = "SELECT em.channel, em.permission"
		." FROM ".$GLOBALS['prefix_fw']."_event_manager as em"
		." JOIN ".$GLOBALS['prefix_fw']."_event_class as ec"
		." WHERE ec.idClass = em.idClass "
		."   AND ec.class='".$event_class."'";

	$rs_manager = mysql_query( $query );
	if( $rs_manager === FALSE )
		return array();
	if( mysql_num_rows($rs_manager) == 0 ) 
		return array();
		
	list($channel, $permission) = mysql_fetch_row( $rs_manager );
	
	$media = array();
	
	if( $permission == 'not_used' ) {
		return array();
	} elseif( $permission == 'mandatory' ) {
		
		if( strlen($channel) > 0 ) {
			$media = explode( ',', $channel );
		}
	} else {
		
		$query = "SELECT eu.channel "
				." FROM ".$GLOBALS['prefix_fw']."_event_user as eu"
				." JOIN ".$GLOBALS['prefix_fw']."_event_manager as em"
				." JOIN ".$GLOBALS['prefix_fw']."_event_class as ec"
				." WHERE eu.idEventMgr=em.idEventMgr "
				."   AND ec.idClass = em.idClass "
				."   AND eu.idst='".(int)$idst."'"
				."   AND ec.class='".$event_class."'";
		$rs_user = mysql_query( $query );
		if( mysql_num_rows($rs_user) == 1 ) {
			list($channel) = mysql_fetch_row( $rs_user );

			if( strlen($channel) > 0 ) {
				$media = explode( ',', $channel );
			}
		} else {
			list($channel) = mysql_fetch_row( $rs_manager );
			if( strlen($channel) > 0 ) {
				$media = explode( ',', $channel );
			}
		}
		return $media;
	}
	return $media;
}

function usernotifier_getUserEventStatus( $idst, $event_class ) {
	$media = usernotifier_getUserEventChannel( $idst, $event_class );
	return (count($media) > 0);
}

?>
