<?php

/************************************************************************/
/* DOCEBO CORE - Framework												*/
/* ============================================							*/
/*																		*/
/* Copyright (c) 2005													*/
/* http://www.docebo.com												*/
/*																		*/
/* This program is free software. You can redistribute it and/or modify	*/
/* it under the terms of the GNU General Public License as published by	*/
/* the Free Software Foundation; either version 2 of the License.		*/
/************************************************************************/

/**
 * @package  DoceboCore
 * @version  $Id: class.field.php 113 2006-03-08 18:08:42Z ema $
 * @category Field
 * @author   Fabio Pirovano <fabio@docebo.com>
 */

/**
 * ABSTRACT class for field implementation
 **/

class Field {
	/**
	 * @var int	$id 	contains the question identifier
	 *
	 * @access public
	 */
	var $id_common;

	/** @var string $field_entry_table the field entry table name */
	var $field_entry_table;

	/**
	 * class constructor
	 */
	function Field($id_common) {
		$this->field_entry_table = $GLOBALS['prefix_fw'].'_field_userentry';
		$this->id_common = $id_common;
	}

	/**
	 * this function is useful for field recognize
	 *
	 * @return string	return the identifier of the field
	 *
	 * @access public
	 */
	function getFieldType() {
		return 'field';
	}

	/**
	 * function to generate filter field xhtml id
	 *
	 * @param string	$id_field		id of the field
	 * @param string 	$field_prefix	(optional) prefix to make id
	 * @return string	return the id of the field in filters
	 *
	 * @access public
	 **/
	 function getFieldId_Filter($id_field, $field_prefix = FALSE) {
		if( $field_prefix === FALSE )
		 	return 'field_filter_'.$id_field;
		else
			return $field_prefix.'field_filter_'.$id_field;
	 }

	/**
	 * function to generate filter field xhtml name
	 *
	 * @param string	$field_id		id of the field
	 * @param string 	$field_prefix	(optional) prefix to make name
	 * @return string	return the name of the field in filters
	 *
	 * @access public
	 **/
	 function getFieldName_Filter($id_field, $field_prefix = FALSE) {
		if( $field_prefix === FALSE )
		 	return 'field_filter['.$id_field.']';
		else
			return $field_prefix.'[field_filter]['.$id_field.']';
	 }

	/**
	 * function to get value of a filter field
	 *
	 * @param array		$array_values	the array to scan for search value
	 * @param string	$id_field		id of the field
	 * @param string 	$field_prefix	(optional) prefix of the field
	 * @return mixed	return the value of the field in filters
	 *
	 * @access public
	 **/
	function getFieldValue_Filter( $array_values, $id_field, $field_prefix = FALSE, $default_value = '' ) {
		if( $field_prefix !== NULL ) {
			if( isset( $array_values[$field_prefix] ) )
				$array_values = $array_values[$field_prefix];
			else
				return $default_value;
		}
		if( isset( $array_values['field_filter'])
			&& isset( $array_values['field_filter'][$id_field]) )
			return $array_values['field_filter'][$id_field];
		else
			return $default_value;
	}

	/**
	 * function to get values of a array of filter field
	 *
	 * @param array		$array_values	the array to scan for search value
	 * @param array		$arr_field_id	array of id of the fields (the keys)
	 * @param string 	$field_prefix	(optional) prefix of the field
	 * @param mixed 	$skipchar 		(optional) if is a number skip the first
	 *									$skipchar char in $arr_field_id search
	 *									if is a string remove all char to the left
	 *									of given string in $arr_field_id search
	 * @return mixed	return the value of the field in filters
	 *
	 * @access public
	 **/
	function getArrFieldValue_Filter( $array_values, $arr_field_id, $field_prefix = FALSE, $skipchar = 0) {
		$result = array();
		if( $field_prefix !== FALSE ) {
			if( isset( $array_values[$field_prefix] ) )
				$array_values = $array_values[$field_prefix];
			else
				return $result;
		}
		if( isset( $array_values['field_filter']) ) {
			foreach( $array_values['field_filter'] as $fname => $fval ) {
				if( is_numeric( $skipchar ) )
					$search_key = substr( $fname, $skipchar );
				else {
					$pos = strpos($fname,$skipchar);
					if( $pos !== FALSE )
						$search_key = substr( $fname, $pos+1 );
					else
						$search_key = $fname;
				}
				if( isset($arr_field_id[$search_key]) ) {
					$result[$fname] = $arr_field_id[$search_key];
					$result[$fname]['value'] = $fval;
				}
			}
		}
		return $result;
	}

	/**
	 * @return string	the main table for database save
	 *
	 * @access private
	 */
	function _getMainTable() {
		return $GLOBALS['prefix_fw'].'_field';
	}

	/**
	 * @return string	the lement table for database save
	 *
	 * @access private
	 */
	function _getElementTable() {
		return $GLOBALS['prefix_fw'].'_field_son';
	}

	/**
	 * @return string	the main table for database user entry save
	 *
	 * @access private
	 */
	function _getUserEntryTable() {
		return $this->field_entry_table;
	}

	/**
	 * Set the field entry table
	 * @param string $field_entry_table the name of the table
	 * @access public
	**/
	function setFieldEntryTable($field_entry_table) {
		$this->field_entry_table = $field_entry_table;
	}

	/**
	 * this function create a new field for future use
	 *
	 * @param  string	$back	indicates the return url
	 * @return nothing
	 *
	 * @access public
	 */
	function create($back) {

	}

	/**
	 * this function manage a field
	 *
	 * @param  string	$back	indicates the return url
	 * @return nothing
	 *
	 * @access public
	 */
	function edit( $back ) {

	}

	/**
	 * this function completely remove a field
	 *
	 * @param  string	$back	indicates the return url
	 * @return nothing
	 *
	 * @access public
	 */
	function deleteUserEntry($id_user) {

		$query_del = "
		DELETE FROM ".$this->_getUserEntryTable()."
		WHERE id_common = '".(int)$this->id_common."' AND id_user = '".(int)$this->id_common."'";
		$re = mysql_query($query_del);
		return $re;
	}

	/**
	 * this function completely remove a field
	 *
	 * @param  string	$back	indicates the return url
	 * @return nothing
	 *
	 * @access public
	 */
	function del($back) {

		$query_del = "
		DELETE FROM ".$this->_getUserEntryTable()."
		WHERE id_common = '".(int)$this->id_common."'";
		$re = mysql_query($query_del);
		doDebug($query_del);
		if(!$re) jumpTo($back.'&result=fail_del');

		$query_del = "
		DELETE FROM ".$this->_getMainTable()."
		WHERE id_common = '".(int)$this->id_common."'";
		$re = mysql_query($query_del);
		doDebug($query_del);

		jumpTo($back.'&result='.( $re ? 'success' : 'fail_del'));
	}

	/**
	 * display the entry of this field for the passed user
	 *
	 * @param 	int		$id_user 			if alredy exists a enty for the user load it
	 *
	 * @return string 	of field xhtml code
	 *
	 * @access public
	 */
	function show( $id_user ) {

		return '';
	}

	/**
	 * display the field for interaction
	 *
	 * @param 	int		$id_user			if alredy exists a entry for the user load as default value
	 * @param 	bool	$freeze				if true, disable the user interaction
	 * @param 	bool	$mandatory			if true, the field is considered mandatory
	 *
	 * @return string 	of field xhtml code
	 *
	 * @access public
	 */
	function play( $id_user, $freeze, $mandatory = false, $value = NULL) {

		return '';
	}

	/**
	 * display the field for filters
	 *
	 * @param	string	$field_id		the id of the field used for id/name
	 * @param 	mixed 	$value 			(optional) the value to put in the field
	 *										retrieved from $_POST if not given
	 * @param	string	$label			(optional) the label to use if not given the
	 *									value will be retrieved from custom field
	 *									$id_field
	 * @param	string	$field_prefix 	(optional) the prefix to give to
	 *									the field id/name
	 * @param 	string 	$other_after 	optional html code added after the input element
	 * @param	string 	$other_before 	optional html code added before the label element
	 * @param   mixed 	$field_special	(optional) special param used in some field type
	 *									see documentation in specific field type
	 *
	 * @return string 	of field xhtml code
	 *
	 * @access public
	 */
	function play_filter( $id_field, $value = FALSE, $label = FALSE, $field_prefix = FALSE, $other_after = '', $other_before = '', $field_special = FALSE ) {

		return '';
	}


	/**
	 * check if the user as selected a valid value for the field
	 *
	 * @return 	bool 	true if operation success false otherwise
	 *
	 * @access public
	 */
	function isFilled( $id_user ) {

		return true;
	}

	/**
	 * return the filled value of the selected field
	 *
	 * @param 	mixed 	$grab_from 			(optional) the array to retrieve the value from
	 *	($_POST will be used as default)
	 *
	 * @return 	bool 	true if operation success false otherwise
	 *
	 * @access public
	 */
	function getFilledVal($grab_from=FALSE) {

		if ($grab_from === FALSE)
			$grab_from=$_POST;

		if(isset($grab_from['field_'.$this->getFieldType()][$this->id_common]))
			return $grab_from['field_'.$this->getFieldType()][$this->id_common];
		else
			return NULL;
	}

	/**
	 * store the value inserted by a user into the database, if a entry exists it will be overwrite
	 *
	 * @param	int		$id_user 		the user
	 * @param	int		$no_overwrite 	if a entry exists do not overwrite it
	 *
	 * @return 	bool 	true if operation success false otherwise
	 *
	 * @access public
	 */
	function store( $id_user, $no_overwrite ) {

		return true;
	}

	/**
	 * store the value passed into the database, if a entry exists it will be overwrite
	 *
	 * @param	int		$id_user 		the user
	 * @param	int		$value 			the value of the field
	 * @param	bool	$is_id 			if false the param must be reconverted
	 * @param	int		$no_overwrite 	if a entry exists do not overwrite it
	 *
	 * @return 	bool 	true if success false otherwise
	 *
	 * @access public
	 */
	function storeDirect( $id_user, $value, $is_id, $no_overwrite ) {

		return true;
	}
	
	/**
	 * use only for special operation
	 *
	 * @access public
	 */
	function specialop() {


	}

	function movetoposition($new_position) {

		$query_del = "
		UPDATE ".$this->_getMainTable()."
		SET sequence = '".$new_position."'
		WHERE id_common = '".(int)$this->id_common."'";
		return mysql_query($query_del);
	}

}

?>