<?php

/************************************************************************/
/* DOCEBO LMS - Learning Managment System                               */
/* ============================================                         */
/*                                                                      */
/* Copyright (c) 2004                                                   */
/* http://www.docebo.com                                                */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/************************************************************************/


class CourseSubscribe_Management {
	
	var $course_man;
	
	var $acl;
	
	var $acl_man;
	
	function CourseSubscribe_Management() {
		
		require_once($GLOBALS['where_lms'].'/lib/lib.course.php');
		require_once($GLOBALS['where_lms'].'/lib/lib.levels.php');
		
		$this->course_man 	= new Man_Course();
		$this->acl 			=& $GLOBALS['current_user']->getAcl();
		$this->acl_man 		=& $GLOBALS['current_user']->getAclManager();
	}
	
	/**
	 * Subscribe a group of users to a course
	 * @param array 	$arr_user 		the id of the users
	 * @param array 	$arr_course 	the id of the courses
	 * @param mixed 	$levels 		a matrix defined in this way 
	 *									array( id_course => array( id_user => lv_number, ... ), ... )
	 *									or else a level_number that is used for all the users
	 *
	 * @return bool true if success, false otherwise
	 */ 
	function multipleSubscribe($arr_users, $arr_courses, $levels) {
		
		$re = true;
		while(list(, $id_course) = each($arr_courses)) {
			
			$re &= $this->subscribeUsers($arr_users , $id_course, ( is_array($levels) ? $levels[$id_course] : $levels ));
		}
		return $re;
	}
	
	/**
	 * Subscribe a group of users to a course
	 * @param array 	$arr_user 	the id of the users
	 * @param int 		$id_course 	the id of the course
	 * @param mixed 	$levels 		a array defined in this way 
	 *									array( id_user => lv_number, ... )
	 *									or else a level_number that is used for all the users
	 *
	 * @return bool true if success, false otherwise
	 */ 
	function subscribeUsers($arr_users, $id_course, $levels) {
		
		if(empty($arr_users)) return true;
		
		$re = true;
		$group_levels 	=& $this->course_man->getCourseIdstGroupLevel($id_course);
		$user_level 	= $this->course_man->getLevelsOfUsers($id_course, $arr_users);
		
		while(list(, $id_user) = each($arr_users)) {
			
			$lv = ( is_array($levels) ? $levels[$id_user] : $levels );
			if(!isset($user_level[$id_user])) {
				
				$this->acl_man->addToGroup($group_levels[$lv], $id_user);
				
				echo $query = "INSERT INTO ".$GLOBALS['prefix_lms']."_courseuser 
				( idUser, idCourse, level, waiting, subscribed_by ) VALUES 
				( '".$id_user."', '".$id_course."', '".$lv."', '0', '".getLogUserId()."' )";
				
				$re &= mysql_query($query);
			} else {
				
				$old_lv = $user_level[$id_user];
				
				$this->acl_man->removeFromGroup($group_levels[$old_lv], $id_user);
				$this->acl_man->addToGroup($group_levels[$lv], $id_user);
				
				echo $query = "
				UPDATE ".$GLOBALS['prefix_lms']."_courseuser 
				SET level = '".$lv."' 
				WHERE idUser = '".$id_user."' AND 
						idCourse = '".$id_course."'";
				$re &= mysql_query($query);
			}
		}
		
		return $re;
	}
	
	/**
	 * Subscribe a user to a course
	 * @param int 	$id_user 		the id of the user
	 * @param int 	$id_course 		the id of the course
	 * @param int 	$level_number 	the level number of the user
	 *
	 * @return bool true if success, false otherwise
	 */ 
	function subscribeUser($id_user, $id_course, $level_number) {
		
		return $this->subscribeUsers(array($id_user), $id_course, $level_number);
	}
	
	/**
	 * Unsubscribe a group of users to a course
	 * @param array 	$arr_user 		the id of the users
	 * @param array 	$arr_course 	the id of the courses
	 *
	 * @return bool true if success, false otherwise
	 */ 
	function multipleUnsubscribe($arr_users, $arr_courses) {
		
		$re = true;
		while(list(, $id_course) = each($arr_courses)) {
			
			$re &= $this->unsubscribeUsers($arr_users , $id_course);
		}
		return $re;
	}
	
	/**
	 * Unsubscribe a group of users to a course
	 * @param array 	$arr_user 	the id of the users
	 * @param int 		$id_course 	the id of the course
	 *
	 * @return bool true if success, false otherwise
	 */ 
	function unsubscribeUsers($arr_users, $id_course) {
		
		if(empty($arr_users)) return true;
		
		$group_levels =& $this->course_man->getCourseIdstGroupLevel($id_course);
		$user_level = $this->course_man->getLevelsOfUsers($id_course, $arr_users);
		
		while(list(, $id_user) = each($arr_users)) {
			
			if(isset($user_level[$id_user])) {
				$lv = $user_level[$id_user];
				$this->acl_man->removeFromGroup($group_levels[$lv], $id_user);
			}
		}
		$re = mysql_query("
		DELETE FROM ".$GLOBALS['prefix_lms']."_courseuser 
		WHERE idUser IN ( ".implode(',', $arr_users)." ) AND idCourse = '".$id_course."'");
		
		return $re;
	}
	
	/**
	 * Unsubscribe a user to a course
	 * @param int 	$id_user 	the id of the user
	 * @param int 	$id_course 	the id of the course
	 *
	 * @return bool true if success, false otherwise
	 */ 
	function unsubscribeUser($id_user, $id_course) {
		
		return $this->unsubscribeUsers(array($id_user), $id_course);
	}
	
	/**
	 * Unsubscribe a user from all the courses
	 * @param int 	$id_user 	the id of the user
	 *
	 * @return bool true if success, false otherwise
	 */ 
	function unsubscribeUserFromAllCourses($id_user) {
		
		$re = mysql_query("
		SELECT idCourse 
		FROM ".$GLOBALS['prefix_lms']."_courseuser 
		WHERE idUser = '".$id_user."'");
		
		$res = true;
		while(list($id_course) = mysql_fetch_row($re)) {
			
			$res &= $this->unsubscribeUsers(array($id_user), $id_course);
		}
		return $res;
	}
}

?>
