/*************************************************************************/
/* SPAGHETTILEARNING - E-Learning System                                 */
/* ============================================                          */
/*                                                                       */
/* Copyright (c) 2004 by Emanuele Sandri (esandri@tiscali.it)            */
/*                                                                       */
/* This program is free software. You can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation; either version 2 of the License.        */
/*************************************************************************/
/*
 * @module ScormApi.js
 * javascript SCORM API 1.2
 * @author Emanuele Sandri
 * @date	12/18/2003
 * @revision 1.1
 *	this revision is based on SOAP protocol specs.
 * @revision 1.2
 *	added event block during communication in geko
 *	added dialog box during communication on IE
 * @version $Id: ScormApi.js 229 2006-04-10 11:35:05Z ema $
 */
/*
 *  - Optimization
 *  Now the objects for connection (spsoap, xmlhttp) are cached as member of
 *  ScormApi object.
 */

 /**
  * class ScormAPI
  * @param baseurl the url to be used to contact the soap interface
  *	@param serviceid the service id to map correct service (hostname:port#SOAPLMS)
  *	@param userid the id of the user
  *	@param scormid id of scorm object
  */
 function ScormApi(host, baseurl, serviceid, idUser, idReference, idscorm_organization) {
 	this.id = "";
	this.idscorm_item = "";
	this.idUser = idUser || "";
	this.idReference = idReference || "";
	this.idscorm_organization = idscorm_organization || "";
	this.xmldoc = null;
	this.errorCode = "";
	this.diagnostic = "";
	this.setErrorCodes;
	this.baseurl = baseurl || "";
	this.host = host || "";
	this.serviceid = serviceid || "";
	this.initialize_cb = null;
	this.finish_cb = null;
	this.getvalue_cb = null;
	this.setvalue_cb = null;
	this.commit_cb = null;
	this.transmission_start_cb = null;
	this.transmission_end_cb = null;
	this.dbgLevel = 0;
	this.dbgOut = null;
	this.initialized = false;
	this.basepath = this.baseurl.substring(0, this.baseurl.lastIndexOf( "/" ));
 }
 
 ScormApi.prototype.setIdscorm_item = function( idscorm_item ) {
 	this.idscorm_item = idscorm_item;
 }
 
 ScormApi.prototype.setIdscorm_organization = function( idscorm_organization ) {
 	this.idscorm_organization = idscorm_organization;
 }
 /*
  * This function indicates to the API Adapter that the SCO is
  * going to communicate with the LMS. It allows the LMS to handle LMS
  * specific initialization issues. It is a requirement of the SCO that
  * it call this function before calling any other API functions.
  */
 ScormApi.prototype.LMSInitialize = function( param ) {
 	this.initialized = true;
 	// @todo: test the connection with LMS and the prerequisites
	if( this.dbgLevel > 0 ) {
		this.dbgPrint( '+LMSInitialize( "' + param + '" );' );
		this.dbgPrint( '-LMSInitialize:"true"' );
	}
	return "true";
 }
 
 ScormApi.prototype.LMSFinish = function( param ) {
 	this.initialized = false;
 	// nothing to do
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '+LMSFinish("' + param + '"); [' + this.idUser + ',' + this.idReference + ',' + this.idscorm_item + ']' );
 	
 	this.resetError();
	var result = "";
	if( this.transmission_start_cb != null )
		this.transmission_start_cb();
	result = this.commonLMSFinish(); 
	if( this.transmission_end_cb != null )
		this.transmission_end_cb();
	
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '-LMSFinish:"' + result + '"' );
	
	if( this.finish_cb != null ) 
		this.finish_cb( param );
	return result;
 }
 
 ScormApi.prototype.LMSCommit = function( param ) {
	if( this.dbgLevel > 0 ) {
		this.dbgPrint( '+LMSCommit( "' + param + '" );' );
		this.dbgPrint( '-LMSCommit:"true"' );
	}
 	return "true";
 }
 
 ScormApi.prototype.LMSGetLastError = function() {
	if( this.dbgLevel > 0 ) {
		this.dbgPrint( '+LMSGetLastError();' );
		this.dbgPrint( '-LMSGetLastError:"' + this.errorCode + '"' );
	}
 	return this.errorCode;
 }

 ScormApi.prototype.LMSGetErrorString = function( ecode ) {
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '+LMSGetErrorString("' + ecode + '");' );
		
 	var result = "";
 	if( this.errorTable[ecode] != null ) {
		result = this.errorTable[ecode];
	} 
	if( this.dbgLevel > 0 ) {
		this.dbgPrint( '-LMSGetErrorString:diagnostic"' + this.diagnostic + '"' );
		this.dbgPrint( '-LMSGetErrorString:"' + result + '"' );
	}
	return result;

 }
 
 ScormApi.prototype.LMSGetDiagnostic = function( ecode ) {
	if( this.dbgLevel > 0 ) {
		this.dbgPrint( '+LMSGetDiagnostic("' + ecode + '");' );
		this.dbgPrint( '+LMSGetDiagnostic:"' + this.diagnostic + '"' );
	}
	
 	return this.diagnostic;
 }
 
 ScormApi.prototype.LMSGetValue = function( param ) {
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '+LMSGetValue("' + param + '"); [' + this.idUser + ',' + this.idReference + ',' + this.idscorm_item + ']' );
 	
 	this.resetError();
	var result = "";
	if( this.transmission_start_cb != null )
		this.transmission_start_cb();
	result = this.commonLMSGetValue( param );
	if( this.transmission_end_cb != null )
		this.transmission_end_cb();
	
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '-LMSGetValue:"' + result + '"' );
		
	return result;
 }

 ScormApi.prototype.LMSSetValue = function( param, data ) {
	if( this.dbgLevel > 0 )
		this.dbgPrint( '+LMSSetValue("' + param + '", "' + data + '");' );

 	this.resetError();
	var result = "";
	if( this.transmission_start_cb != null )
		this.transmission_start_cb();
	result = this.commonLMSSetValue( param, data );
	if( this.transmission_end_cb != null )
		this.transmission_end_cb();
	
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '-LMSSetValue:"' + result + '"' );
	return result;
 }
  
 // =========== Private functions ============================

ScormApi.prototype.commonLMSGetValue = function( param ) {
    if( this['xmlhttp'] == null ) {
		this.xmlhttp = this.CreateXmlHttpRequest();
	}
	var xmlhttp = this.xmlhttp;
	
	xmlhttp.open("POST", this.baseurl, false);
	
	xmlhttp.setRequestHeader("Man", "POST " + this.baseurl + " HTTP/1.1");
	xmlhttp.setRequestHeader("Host", this.host );
	xmlhttp.setRequestHeader("Content-Type", "text/xml; charset=utf-8");
	xmlhttp.setRequestHeader("SOAPAction", this.serviceid + "GetValue" );

	strSoap = '<?xml version="1.0" encoding="utf-8"?>' 
			+ '<env:Envelope xmlns:env="http://schemas.xmlsoap.org/soap/envelope/"'
			+ ' xmlns:enc="http://schemas.xmlsoap.org/soap/encoding/"' 
			+ ' env:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"'
			+ ' xmlns:xs="http://www.w3.org/1999/XMLSchema"'
			+ ' xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">'
			+	'<env:Header/>'
			+	'<env:Body>'
			+		'<a0:GetValue xmlns:a0="' + this.serviceid + '">'
			+			'<idUser xsi:type="xs:string">' + this.idUser + '</idUser>'
			+			'<idReference xsi:type="xs:string">' + this.idReference + '</idReference>'
			+			'<idscorm_item xsi:type="xs:string">' + this.idscorm_item + '</idscorm_item>'
			+			'<param xsi:type="xs:string">' + param + '</param>'
			+		'</a0:GetValue>'
			+	'</env:Body>'
			+ '</env:Envelope>';
	try {
	xmlhttp.send(strSoap);
	} catch( ex1 ) {
   		alert( ex1 );		
	}
	if( xmlhttp.status == 200 ) {
		try {
			var xmldoc = xmlhttp.responseXML;		
			var status = xmldoc.getElementsByTagName("status").item(0).firstChild;
			var errorCode = xmldoc.getElementsByTagName("error").item(0).firstChild;
			var errorText = xmldoc.getElementsByTagName("errorString").item(0).firstChild;
			var value = xmldoc.getElementsByTagName("value").item(0).firstChild;		
			if( status.nodeValue == "success" ) {
				return ((value == null)?"":value.nodeValue);
			} else {
				this.setError(errorCode == null ? "102":errorCode.nodeValue);
				this.diagnostic = errorText == null ? "":errorText.nodeValue;
				return "";
			}	
		} catch (ex) {
			w = window.open('#', 'debug');
			w.document.open();
			w.document.write( xmlhttp.responseText );
			w.document.close();
   			alert( ex );
			alert( xmlhttp.responseText );
		}
	} else {
		this.setError("101");
		this.diagnostic = xmlhttp.responseText;
		alert( this.diagnostic );
	}
}

ScormApi.prototype.commonLMSSetValue = function( param, value ) {
    if( this['xmlhttp'] == null ) {		
		this.xmlhttp = this.CreateXmlHttpRequest();
	}
	var xmlhttp = this.xmlhttp;
	if( this.dbgLevel > 0 ) {
	    this.dbgPrint( 'ieLMSSetValue info:"' + this.baseurl + '"' );
	}
	xmlhttp.open("POST", this.baseurl, false);
	
	xmlhttp.setRequestHeader("Man", "POST " + this.baseurl + " HTTP/1.1");
	xmlhttp.setRequestHeader("Host", this.host );
	xmlhttp.setRequestHeader("Content-Type", "text/xml; charset=utf-8");
	xmlhttp.setRequestHeader("SOAPAction", this.serviceid + "SetValue" );

	strSoap = '<?xml version="1.0" encoding="utf-8"?>' 
			+ '<env:Envelope xmlns:env="http://schemas.xmlsoap.org/soap/envelope/"'
			+ ' xmlns:enc="http://schemas.xmlsoap.org/soap/encoding/"'
			+ ' env:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"'
			+ ' xmlns:xs="http://www.w3.org/1999/XMLSchema"'
			+ ' xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">'
			+ 	'<env:Header/>'
			+ 	'<env:Body>'
			+		'<a0:SetValue xmlns:a0="' + this.serviceid + '">'
			+			'<idUser xsi:type="xs:string">' + this.idUser + '</idUser>'
			+			'<idReference xsi:type="xs:string">' + this.idReference + '</idReference>'
			+			'<idscorm_item xsi:type="xs:string">' + this.idscorm_item + '</idscorm_item>'
			+			'<param xsi:type="xs:string">' + param + '</param>'
			+			'<value xsi:type="xs:string">' + value + '</value>'
			+		'</a0:SetValue>'
			+	'</env:Body>'
			+ '</env:Envelope>';

			
	xmlhttp.send(strSoap);
	
	if( xmlhttp.status == 200 ) {
		var xmldoc = xmlhttp.responseXML;
		var status = xmldoc.getElementsByTagName("status").item(0).firstChild;
		var errorCode = xmldoc.getElementsByTagName("error").item(0).firstChild;
		var errorText = xmldoc.getElementsByTagName("errorString").item(0).firstChild;
		if( status.nodeValue == "success" ) {
			return "true";
		} else {
			this.setError(errorCode == null ? "102":errorCode.nodeValue);
			this.diagnostic = errorText == null ? "":errorText.nodeValue;
			return "false";
		}		
	} else {
		this.setError("101");
		this.diagnostic = xmlhttp.responseText;
		return "false";
	}
} 

ScormApi.prototype.commonLMSFinish = function( param ) {
    if( this['xmlhttp'] == null ) {
		this.xmlhttp = this.CreateXmlHttpRequest();
	}
	var xmlhttp = this.xmlhttp;
	
	xmlhttp.open("POST", this.baseurl, false);
	
	xmlhttp.setRequestHeader("Man", "POST " + this.baseurl + " HTTP/1.1");
	xmlhttp.setRequestHeader("Host", this.host );
	xmlhttp.setRequestHeader("Content-Type", "text/xml; charset=utf-8");
	xmlhttp.setRequestHeader("SOAPAction", this.serviceid + "Finish" );

	//xmlhttp.responseXML.async = false;	
	
	strSoap = '<?xml version="1.0" encoding="utf-8"?>' 
			+ '<env:Envelope xmlns:env="http://schemas.xmlsoap.org/soap/envelope/"'
			+ ' xmlns:enc="http://schemas.xmlsoap.org/soap/encoding/"' 
			+ ' env:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"'
			+ ' xmlns:xs="http://www.w3.org/1999/XMLSchema"'
			+ ' xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">'
			+	'<env:Header/>'
			+	'<env:Body>'
			+		'<a0:Finish xmlns:a0="' + this.serviceid + '">'
			+			'<idUser xsi:type="xs:string">' + this.idUser + '</idUser>'
			+			'<idReference xsi:type="xs:string">' + this.idReference + '</idReference>'
			+			'<idscorm_item xsi:type="xs:string">' + this.idscorm_item + '</idscorm_item>'
			+		'</a0:Finish>'
			+	'</env:Body>'
			+ '</env:Envelope>';
	
	//alert( strSoap );
	//return;
	xmlhttp.send(strSoap);
	
	if( xmlhttp.status == 200 ) {
		try {
			var xmldoc = xmlhttp.responseXML;		
			var status = xmldoc.getElementsByTagName("status").item(0).firstChild;
			var errorCode = xmldoc.getElementsByTagName("error").item(0).firstChild;
			var errorText = xmldoc.getElementsByTagName("errorString").item(0).firstChild;
			if( status.nodeValue == "success" ) {
				return "true";
			} else {
				this.setError(errorCode == null ? "102":errorCode.nodeValue);
				this.diagnostic = errorText == null ? "":errorText.nodeValue;
				return "false";
			}	
		} catch (ex) {
			w = window.open('#', 'debug');
			w.document.open();
			w.document.write( xmlhttp.responseText );
			w.document.close();
   			alert( ex );
			return "false"
		}
	} else {
		this.setError("101");
		this.diagnostic = xmlhttp.responseText;
		return "false"
	}
}

ScormApi.prototype.CreateXmlHttpRequest = function() {
	// try first IE
	var xmlhttp = false;
	var actXArray = new Array( 	"MSXML4.XmlHttp", "MSXML3.XmlHttp", "MSXML2.XmlHttp", 
								"MSXML.XmlHttp", "Microsoft.XmlHttp" );
	var created = false;
	for( var i = 0; i < actXArray.length && !created; i++ ) {
		try {
			xmlhttp=new ActiveXObject(actXArray[i]);
			created = true;
		} catch (e) {
			// do nothing
		}
	}
	if (!xmlhttp) {
		try {
			xmlhttp = new XMLHttpRequest();
		} catch (e) {
			xmlhttp=false;
		}
	}
	if( !xmlhttp ) 
		alert( "This browser don't support required functionalities for scorm" );
	return xmlhttp;
}
 
 /**
  *	@internal
  * Set the error code 
  * @param string ecode error code
  * @return null
  */
 ScormApi.prototype.setError = function(ecode) {
 	this.errorCode = ecode;
 }
 
  /**
  *	@internal
  * Reset the error code 
  * @return null
  */
 ScormApi.prototype.resetError = function() {
 	this.errorCode = "";
 }

 /**
  * @interal
  *	Error table
  */
 ScormApi.prototype.SCORM_STATUS_SUCCESS = "Success";
 ScormApi.prototype.SCORM_STATUS_ERROR = "Error";
 ScormApi.prototype.errorTable = new Object();
 ScormApi.prototype.errorTable["0"] = "No error";
 ScormApi.prototype.errorTable["101"] = "General exception";
 ScormApi.prototype.errorTable["201"] = "Invalid argument error";
 ScormApi.prototype.errorTable["202"] = "Elemet cannot have children";
 ScormApi.prototype.errorTable["203"] = "Elemet not an array - cannot have count";
 ScormApi.prototype.errorTable["301"] = "Not initialized";
 ScormApi.prototype.errorTable["401"] = "Not implemented error";
 ScormApi.prototype.errorTable["402"] = "Invalid set value, element is a keyword";
 ScormApi.prototype.errorTable["403"] = "Element is read only";
 ScormApi.prototype.errorTable["404"] = "Element is write only";
 ScormApi.prototype.errorTable["405"] = "Incorrect data type";

 /**
  *	@internal
  * Scorm LMS path
  */
 ScormApi.prototype.LMSUrl = "";
 
 
 /**
  * @internal
  * Debug print out function
  */
ScormApi.prototype.dbgPrint = function( text ) {
	var doc = this.dbgOut.ownerDocument;
	var outelem = doc.createTextNode('[' + this.userid + ',' + this.scoid + ',' + this.idscormpackage + '] ' + text);
	var crelem = doc.createElement("BR");
	this.dbgOut.appendChild(outelem);
	this.dbgOut.appendChild(crelem);
}

function ScormApiUI( host, baseurl, serviceid, idUser, idReference, idscorm_organization ) {
	this.base = ScormApi;
	this.base(host, baseurl, serviceid, idUser, idReference, idscorm_organization);
	this.slEventCap = 0;
	this.sStyle = "dialogHeight:100px;dialogWidth:150px";
}

ScormApiUI.prototype = new ScormApi;
ScormApiUI.prototype.LMSGetValue = function( param ) {
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '+LMSGetValue("' + param + '"); [' + this.idUser + ',' + this.idReference + ',' + this.idscorm_item + ']' );
 	
 	this.resetError();
	var result = "";
	if( this.transmission_start_cb != null )
		this.transmission_start_cb();
	
	if( window.document.all && false ) {	//IE
		// compute page position
		var args = new Object();
		args.sapi = this;
		args.func = "GetValue";
		args.param = param;
		result = showModalDialog( this.basepath + "/dialog.php", args, this.sStyle );
	} else {
		slStopEvents(window.top);
		result = this.commonLMSGetValue( param );
		window.setTimeout(	function restartEvents() { 
							slStartEvents(window.top); 
						}, 
						300 
					);
	}
	
	if( this.transmission_end_cb != null )
		this.transmission_end_cb();
	
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '-LMSGetValue:"' + result + '"' );
		
	return result;
 }
 
 ScormApiUI.prototype.LMSSetValue = function( param, value ) {
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '+LMSSetValue("' + param + '", "' + data + '");' );
 	
 	this.resetError();
	var result = "";
	if( this.transmission_start_cb != null )
		this.transmission_start_cb();
	
	if( window.document.all && false ) {	//IE
		// compute page position
		var args = new Object();
		args.sapi = this;
		args.func = "SetValue";
		args.param = param;
		args.value = value;
		result = showModalDialog( 	this.basepath + "/dialog.php", 
						args, 
						this.sStyle );
	} else {
		slStopEvents(window.top);
		result = this.commonLMSSetValue( param, value );
		window.setTimeout(	function restartEvents() { 
							slStartEvents(window.top); 
						}, 
						300 
					);
	}
	
	if( this.transmission_end_cb != null )
		this.transmission_end_cb();
	
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '-LMSSetValue:"' + result + '"' );
		
	return result;
 }
 
 ScormApi.prototype.LMSFinish = function( param ) {
 	this.initialized = false;
 	// nothing to do
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '+LMSFinish("' + param + '"); [' + this.idUser + ',' + this.idReference + ',' + this.idscorm_item + ']' );
 	
 	this.resetError();
	var result = "";
	if( this.transmission_start_cb != null )
		this.transmission_start_cb();
	
	if( window.document.all && false ) {	//IE
		// compute page position
		var args = new Object();
		args.sapi = this;
		args.func = "Finish";
		args.param = param;
		result = showModalDialog( this.basepath + "/dialog.php", args, this.sStyle );
	} else {
		slStopEvents(window.top);
		result = this.commonLMSFinish();
		window.setTimeout(	function restartEvents() { 
							slStartEvents(window.top); 
						}, 
						300 
					);
	}
	if( this.transmission_end_cb != null )
		this.transmission_end_cb();
	
	if( this.dbgLevel > 0 ) 
		this.dbgPrint( '-LMSFinish:"' + result + '"' );
	
	if( this.finish_cb != null ) 
		this.finish_cb( param );
	return result;
 }
 
 function eventCanceller(ev) {
	if (!ev) var ev = window.event;
	//alert( ev );
	if( ev.stopPropagation ) {
		ev.preventDefault();        //DOM
		ev.stopPropagation();
	} else {
		ev.cancelBubble = true; //IE
		//alert( ev );
	}
	
	return false;
}

var slEventCap = 0;
// capture some window's events
function capwin(w) {
	if( w.document.attachEvent ) { //IE 5+
		w.document.attachEvent( "onclick",eventCanceller );
		w.document.attachEvent( "onmousedown",eventCanceller );
		w.document.attachEvent( "onfocus",eventCanceller );
	} else if( w.document.all ) { // IE 4
		w.onclick = eventCanceller;
		w.onmousedown = eventCanceller;
		w.onfocus = eventCanceller;
	} else { // geko
		w.addEventListener("click", eventCanceller, true);
		w.addEventListener("mousedown", eventCanceller, true);
		w.addEventListener("focus", eventCanceller, true);
	}
}

// release the captured events
function relwin(w) {
	try {
		if( w.document.attachEvent ) { //IE 5+
			w.document.detachEvent( "onclick", eventCanceller );
			w.document.detachEvent( "onmousedown", eventCanceller );
			w.document.detachEvent( "onfocus", eventCanceller );
		} else if( w.document.all ) { // IE 4
			w.onclick = null;
			w.onmousedown = null;
			w.onfocus = null;
		} else {
			w.removeEventListener("focus", eventCanceller, true);
			w.removeEventListener("mousedown", eventCanceller, true);
			w.removeEventListener("click", eventCanceller, true);
		}
	} catch( ex ) {
		// do nothing
	}
}


function slStopEvents(win) {
	slEventCap++;
	if( slEventCap > 1 ) 
		return;
	capwin(win);
	// capture other frames
	for (var i = 0; i < window.frames.length; capwin(window.frames[i++]));  
}

function slStartEvents(win) {
	slEventCap--;
	if( slEventCap > 0 ) 
		return;
	relwin(win);
	// release other frames
	for (var i = 0; i < window.frames.length; relwin(window.frames[i++])); 
}

